@@ -17,14 +17,29 @@
 
 #include <log4cxx/logstring.h>
 #include <log4cxx/helpers/outputstream.h>
+#include <stdexcept>
 
 using namespace log4cxx;
 using namespace log4cxx::helpers;
 
 IMPLEMENT_LOG4CXX_OBJECT(OutputStream)
 
-OutputStream::OutputStream() {
+OutputStream::OutputStream()
+{
 }
 
-OutputStream::~OutputStream() {
+OutputStream::~OutputStream()
+{
 }
+
+#ifdef LOG4CXX_MULTI_PROCESS
+apr_file_t* OutputStream::getFilePtr()
+{
+	throw std::logic_error("getFilePtr must be implemented in the derived class that you are using");
+}
+
+OutputStream& OutputStream::getFileOutPutStreamPtr()
+{
+	throw std::logic_error("getFileOutPutStreamPtr must be implemented in the derived class that you are using");
+}
+#endif
