@@ -13,10 +13,25 @@
 #include <strings.h>
 #endif
 
+#if defined ZZIP_HAVE_STRNLEN || defined strnlen
+#define _zzip_strnlen strnlen
+#else
+#include <stdlib.h>
+
+/* if your system does not have strnlen: */
+static size_t
+_zzip_strnlen(const char *p, size_t maxlen)
+{
+    const char * stop = (char *)memchr(p, '\0', maxlen);
+    return stop ? (size_t)(stop - p) : maxlen;
+}
+#endif
+
 
 #if defined ZZIP_HAVE_STRNDUP || defined strndup
 #define _zzip_strndup strndup
 #else
+#include <stdlib.h>
 
 /* if your system does not have strndup: */
 zzip__new__ static char *
@@ -27,8 +42,8 @@ _zzip_strndup(char const *p, size_t maxlen)
        return p;
     } else 
     {
-        size_t len = strnlen(p, maxlen);
-        char* r = malloc(len + 1);
+        size_t len = _zzip_strnlen(p, maxlen);
+        char* r = (char *)malloc(len + 1);
         if (r == NULL)
             return NULL; /* errno = ENOMEM */
         r[len] = '\0';
