@@ -37,6 +37,9 @@
 #include <log4cxx/logger.h>
 #include <log4cxx/rollingfileappender.h>
 #include <log4cxx/xml/domconfigurator.h>
+#if defined(LOG4CXX_INFO_FMT)
+#  define MIKTEX_LOG4CXX_12 1
+#endif
 
 #include <miktex/App/Application>
 #include <miktex/Configuration/ConfigNames>
@@ -733,12 +736,17 @@ bool Application::InstallPackage(const string& packageId, const PathName& trigge
     cerr
       << "\n"
       << fmt::format(T_("Unfortunately, the package {0} could not be installed."), packageId) << endl;
-    log4cxx::RollingFileAppenderPtr appender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
-    if (appender != nullptr)
+#if defined(MIKTEX_LOG4CXX_12)
+    log4cxx::AppenderPtr appender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
+    log4cxx::FileAppenderPtr fileAppender = log4cxx::cast<log4cxx::FileAppender>(appender);
+#else
+    log4cxx::FileAppenderPtr fileAppender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
+#endif
+    if (fileAppender != nullptr)
     {
       cerr
         << T_("Please check the log file:") << "\n"
-        << PathName(appender->getFile()) << endl;
+        << PathName(fileAppender->getFile()) << endl;
     }
   }
   if (switchToAdminMode)
@@ -913,14 +921,19 @@ void Application::Sorry(const string& name, const string& description, const str
   }
   if (isLog4cxxConfigured)
   {
-    log4cxx::RollingFileAppenderPtr appender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
-    if (appender != nullptr)
+#if defined(MIKTEX_LOG4CXX_12)
+    log4cxx::AppenderPtr appender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
+    log4cxx::FileAppenderPtr fileAppender = log4cxx::cast<log4cxx::FileAppender>(appender);
+#else
+    log4cxx::FileAppenderPtr fileAppender = log4cxx::Logger::getRootLogger()->getAppender(LOG4CXX_STR("RollingLogFile"));
+#endif
+    if (fileAppender != nullptr)
     {
       cerr
         << "\n"
         << T_("The log file hopefully contains the information to get MiKTeX going again:") << "\n"
         << "\n"
-        << "  " << PathName(appender->getFile())
+        << "  " << PathName(fileAppender->getFile())
         << endl;
     }
   }
