@@ -32,6 +32,7 @@
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 
+#include <miktex/Core/AutoResource>
 #include <miktex/Core/File>
 #include <miktex/Core/FileStream>
 #include <miktex/Core/LockFile>
@@ -62,10 +63,10 @@ void MIKTEXNORETURN ThrowFndbDamaged(const string& description, const MiKTeXExce
   Session::FatalMiKTeXError(T_("The file name database is damaged."), description, T_("Delete the file name database files. Then run 'initexmf -u' to recreate the FNDB."), "fndb-damaged", info, sourceLocation);
 }
 
-shared_ptr<FileNameDatabase> FileNameDatabase::Create(const PathName& fndbPath, const PathName& rootDirectory)
+shared_ptr<FileNameDatabase> FileNameDatabase::Create(const PathName& fndbPath, const PathName& rootDirectory, shared_ptr<FileSystemWatcher> fsWatcher)
 {
   shared_ptr<FileNameDatabase> fndb = make_shared<FileNameDatabase>();
-  fndb->Initialize(fndbPath, rootDirectory);
+  fndb->Initialize(fndbPath, rootDirectory, fsWatcher);
   return fndb;
 }
 
@@ -135,10 +136,10 @@ bool FileNameDatabase::Search(const PathName& relativePath, const string& pathPa
 
   ApplyChangeFile();
 
-  trace_fndb->WriteLine("core", fmt::format(T_("fndb search: rootDirectory={0}, relativePath={1}, pathpattern={2}"), Q_(rootDirectory), Q_(relativePath), Q_(pathPattern)));
+  trace_fndb->WriteLine("core", fmt::format(T_("fndb search: rootDirectory={0}, relativePath={1}, pathPattern={2}"), Q_(rootDirectory), Q_(relativePath), Q_(pathPattern)));
 
   MIKTEX_ASSERT(result.size() == 0);
-  MIKTEX_ASSERT(!PathNameUtil::IsAbsolutePath(relativePath));
+  MIKTEX_ASSERT(!PathNameUtil::IsAbsolutePath(relativePath.GetData()));
   MIKTEX_ASSERT(!IsExplicitlyRelativePath(relativePath.GetData()));
 
   PathName dir = relativePath.GetDirectoryName();
@@ -222,7 +223,7 @@ void FileNameDatabase::Add(const vector<Fndb::Record>& records)
 #if 1
   // TODO: File::Sync API
 #if defined(MIKTEX_WINDOWS)
-  if (!FlushFileBuffers(reinterpret_cast<HANDLE>(_get_osfhandle(fileno(writer.GetFile())))))
+  if (!FlushFileBuffers(reinterpret_cast<HANDLE>(_get_osfhandle(_fileno(writer.GetFile())))))
   {
     MIKTEX_FATAL_WINDOWS_ERROR("FlushFileBuffers");
   }
@@ -255,7 +256,7 @@ void FileNameDatabase::Remove(const vector<PathName>& paths)
 #if 1
   // TODO: File::Sync API
 #if defined(MIKTEX_WINDOWS)
-  if (!FlushFileBuffers(reinterpret_cast<HANDLE>(_get_osfhandle(fileno(writer.GetFile())))))
+  if (!FlushFileBuffers(reinterpret_cast<HANDLE>(_get_osfhandle(_fileno(writer.GetFile())))))
   {
     MIKTEX_FATAL_WINDOWS_ERROR("FlushFileBuffers");
   }
@@ -388,6 +389,11 @@ void FileNameDatabase::ReadFileNames(const FileNameDatabaseRecord* table)
 
 void FileNameDatabase::Finalize()
 {
+  if (fsWatcher != nullptr)
+  {
+    fsWatcher->Unsubscribe(this);
+    fsWatcher = nullptr;
+  }
   if (trace_fndb != nullptr)
   {
     trace_fndb->WriteLine("core", fmt::format(T_("unloading fndb {0}"), Q_(this->rootDirectory)));
@@ -400,22 +406,40 @@ void FileNameDatabase::Finalize()
   }
 }
 
-void FileNameDatabase::Initialize(const PathName& fndbPath, const PathName& rootDirectory)
+void FileNameDatabase::Initialize(const PathName& fndbPath, const PathName& rootDirectory, shared_ptr<FileSystemWatcher> fsWatcher)
 {
   this->rootDirectory = rootDirectory;
 
+  this->fsWatcher = fsWatcher;
+  fsWatcher->Subscribe(this);
+  fsWatcher->AddDirectory(fndbPath.GetDirectoryName());
+
   OpenFileNameDatabase(fndbPath);
   ReadFileNames();
 
   changeFile = fndbPath;
   changeFile.SetExtension(MIKTEX_FNDB_CHANGE_FILE_SUFFIX);
   
+  changeFileModified = true;
   ApplyChangeFile();
 }
 
+void FileNameDatabase::OnChange(const MiKTeX::Core::FileSystemChangeEvent& ev)
+{
+  if (ev.fileName == changeFile && ev.action == FileSystemChangeAction::Modified)
+  {
+    changeFileModified = true;
+  }
+}
+
 void FileNameDatabase::ApplyChangeFile()
 {
   lastAccessTime = chrono::high_resolution_clock::now();
+  if (!changeFileModified)
+  {
+    return;
+  }
+  MIKTEX_AUTO(changeFileModified = false);
   if (!File::Exists(changeFile))
   {
     return;
@@ -434,7 +458,7 @@ void FileNameDatabase::ApplyChangeFile()
   }
   if (changeFileSize > 0)
   {
-    reader.Seek(changeFileSize, SeekOrigin::Begin);
+    reader.Seek(static_cast<long>(changeFileSize), SeekOrigin::Begin);
   }
   for (string line; Utils::ReadLine(line, reader.GetFile(), false); )
   {
