@@ -1,4 +1,4 @@
-/* SessionImpl.h: Session impl class                    -*- C++ -*-
+/* SessionImpl.h: Session impl class
 
    Copyright (C) 1996-2021 Christian Schenk
 
@@ -42,6 +42,7 @@
 #include "core-version.h"
 
 #include <miktex/Core/Cfg>
+#include <miktex/Core/FileSystemWatcher>
 #include <miktex/Core/Session>
 #include <miktex/Core/Stream>
 #include <miktex/Core/equal_icase>
@@ -67,35 +68,30 @@ namespace MiKTeXSessionLib = MAKE_CURVER_ID(MiKTeXSession);
 struct FormatInfo_ :
   public MiKTeX::Core::FormatInfo
 {
-public:
   FormatInfo_()
   {
   }
 
-public:
   FormatInfo_(const MiKTeX::Core::FormatInfo& other) :
     MiKTeX::Core::FormatInfo(other)
   {
   }
 
-public:
   MiKTeX::Util::PathName cfgFile;
 };
 
-struct LanguageInfo_ : public MiKTeX::Core::LanguageInfo
+struct LanguageInfo_ :
+  public MiKTeX::Core::LanguageInfo
 {
-public:
   LanguageInfo_()
   {
   }
 
-public:
-  LanguageInfo_(const MiKTeX::Core::LanguageInfo& other) :
+  LanguageInfo_(const MiKTeX::Core::LanguageInfo &other) :
     MiKTeX::Core::LanguageInfo(other)
   {
   }
 
-public:
   MiKTeX::Util::PathName cfgFile;
 };
 
@@ -115,36 +111,36 @@ inline bool operator<(const LanguageInfo_& lhs, const LanguageInfo_& rhs)
 struct InternalFileTypeInfo :
   public MiKTeX::Core::FileTypeInfo
 {
-public:
-  std::vector<MiKTeX::Util::PathName> searchVec;
+  std::vector<MiKTeX::Util::PathName> pathPatterns;
 };
 
-class DvipsPaperSizeInfo : public MiKTeX::Core::PaperSizeInfo
+struct DvipsPaperSizeInfo :
+  public MiKTeX::Core::PaperSizeInfo
 {
-public:
   std::vector<std::string> definition;
 };
 
-struct VersionedStartupConfig :
+struct InternalStartupConfig :
   public MiKTeX::Core::StartupConfig
 {
-public:
-  VersionedStartupConfig()
+  InternalStartupConfig()
   {
   }
 
-public:
-  VersionedStartupConfig(const MiKTeX::Core::StartupConfig& parent) :
+  InternalStartupConfig(const MiKTeX::Core::StartupConfig& parent) :
     StartupConfig(parent)
   {
   }
   
+  /// Indicates whether the installation is shared by all users.
+  MiKTeX::Configuration::TriState isSharedSetup = MiKTeX::Configuration::TriState::Undetermined;
+
   /// MiKTeX setup version.
-public:
   MiKTeX::Core::VersionNumber setupVersion;
 };
 
-class SessionImpl : public MiKTeX::Core::Session
+class SessionImpl :
+  public MiKTeX::Core::Session
 {
 public:
   SessionImpl();
@@ -329,21 +325,21 @@ public:
   MiKTeX::Core::FileType DeriveFileType(const MiKTeX::Util::PathName& fileName) override;
 
 public:
-  bool FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, std::vector<MiKTeX::Util::PathName>& result) override;
+  bool FindFile(const std::string& fileName, const std::string& searchPath, FindFileOptionSet options, std::vector<MiKTeX::Util::PathName>& result) override;
 
 public:
-  bool FindFile(const std::string& fileName, const std::string& pathList, std::vector<MiKTeX::Util::PathName>& result) override
+  bool FindFile(const std::string& fileName, const std::string& searchPath, std::vector<MiKTeX::Util::PathName>& result) override
   {
-    return FindFile(fileName, pathList, { FindFileOption::All }, result);
+    return FindFile(fileName, searchPath, { FindFileOption::All }, result);
   }
 
 public:
-  bool FindFile(const std::string& fileName, const std::string& pathList, FindFileOptionSet options, MiKTeX::Util::PathName& result) override;
+  bool FindFile(const std::string& fileName, const std::string& searchPath, FindFileOptionSet options, MiKTeX::Util::PathName& result) override;
 
 public:
-  bool FindFile(const std::string& fileName, const std::string& pathList, MiKTeX::Util::PathName& result) override
+  bool FindFile(const std::string& fileName, const std::string& searchPath, MiKTeX::Util::PathName& result) override
   {
-    return FindFile(fileName, pathList, {}, result);
+    return FindFile(fileName, searchPath, {}, result);
   }
 
 public:
@@ -745,10 +741,10 @@ private:
   bool GetWorkingDirectory(unsigned n, MiKTeX::Util::PathName& path);
 
 private:
-  std::vector<MiKTeX::Util::PathName> ConstructSearchVector(MiKTeX::Core::FileType fileType);
+  std::vector<MiKTeX::Util::PathName> GetDirectoryPatterns(MiKTeX::Core::FileType fileType);
 
 private:
-  void TraceSearchVector(const char* key, const std::vector<MiKTeX::Util::PathName>& pathvec);
+  void TraceDirectoryPatterns(const std::string& fileType, const std::vector<MiKTeX::Util::PathName>& pathPatterns);
 
 private:
   void RegisterLibraryTraceStreams();
@@ -768,15 +764,14 @@ private:
 private:
   bool InternalGetFontInfo(const std::string& fontName, std::string& supplier, std::string& typeface);
 
-
 private:
   bool MakePkFileName(MiKTeX::Util::PathName& pkFileName, const std::string& fontName, int dpi);
 
 private:
-  bool FindFileInternal(const std::string& fileName, const std::vector<MiKTeX::Util::PathName>& vec, bool all, bool useFndb, bool searchFileSystem, std::vector<MiKTeX::Util::PathName>& result);
+  bool FindFileInDirectories(const std::string& fileName, const std::vector<MiKTeX::Util::PathName>& pathPatterns, bool all, bool useFndb, bool searchFileSystem, std::vector<MiKTeX::Util::PathName>& result);
 
 private:
-  bool FindFileInternal(const std::string& fileName, MiKTeX::Core::FileType fileType, bool all, bool tryHard, bool create, bool renew, std::vector<MiKTeX::Util::PathName>& result);
+  bool FindFileByType(const std::string& fileName, MiKTeX::Core::FileType fileType, bool all, bool tryHard, bool create, bool renew, std::vector<MiKTeX::Util::PathName>& result);
 
 private:
   bool SearchFileSystem(const std::string& fileName, const char* dirPath, bool all, std::vector<MiKTeX::Util::PathName>& result);
@@ -850,36 +845,42 @@ private:
 private:
   unsigned RegisterRootDirectory(const MiKTeX::Util::PathName& root, MiKTeX::Core::RootDirectoryInfo::Purpose purpose, MiKTeX::Core::ConfigurationScope scope, bool other, bool review);
 
+private:
+  bool FindBinRelative(const MiKTeX::Util::PathName& relPath, MiKTeX::Util::PathName& path);
+
 private:
   bool FindStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, MiKTeX::Util::PathName& path);
 
 private:
-  VersionedStartupConfig ReadStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, const MiKTeX::Util::PathName& path);
+  InternalStartupConfig ReadStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, const MiKTeX::Util::PathName& path);
+
+private:
+  InternalStartupConfig ReadMiKTeXConfig(const MiKTeX::Util::PathName& path);
 
 private:
   MiKTeX::Util::PathName GetStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, MiKTeX::Core::MiKTeXConfiguration config, MiKTeX::Core::VersionNumber version);
 
 private:
-  void WriteStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, const VersionedStartupConfig& startupConfig);
+  void WriteStartupConfigFile(MiKTeX::Core::ConfigurationScope scope, const InternalStartupConfig& startupConfig);
 
 private:
-  VersionedStartupConfig ReadEnvironment(MiKTeX::Core::ConfigurationScope scope);
+  InternalStartupConfig ReadEnvironment(MiKTeX::Core::ConfigurationScope scope);
 
 #if defined(MIKTEX_WINDOWS)
 private:
-  VersionedStartupConfig ReadRegistry(MiKTeX::Core::ConfigurationScope scope);
+  InternalStartupConfig ReadRegistry(MiKTeX::Core::ConfigurationScope scope);
 #endif
 
 #if defined(MIKTEX_WINDOWS)
 private:
-  void WriteRegistry(MiKTeX::Core::ConfigurationScope scope, const VersionedStartupConfig& startupConfig);
+  void WriteRegistry(MiKTeX::Core::ConfigurationScope scope, const InternalStartupConfig& startupConfig);
 #endif
 
 private:
-  VersionedStartupConfig DefaultConfig(MiKTeX::Core::MiKTeXConfiguration config, MiKTeX::Core::VersionNumber setupVersion, const MiKTeX::Util::PathName& commonPrefix, const MiKTeX::Util::PathName& userPrefix);
+  InternalStartupConfig DefaultConfig(MiKTeX::Core::MiKTeXConfiguration config, MiKTeX::Core::VersionNumber setupVersion, const MiKTeX::Util::PathName& commonPrefix, const MiKTeX::Util::PathName& userPrefix);
 
 private:
-  VersionedStartupConfig DefaultConfig()
+  InternalStartupConfig DefaultConfig()
   {
     return DefaultConfig(initStartupConfig.config, initStartupConfig.setupVersion, MiKTeX::Util::PathName(), MiKTeX::Util::PathName());
   }
@@ -888,13 +889,13 @@ private:
   void InitializeStartupConfig();
 
 private:
-  void MergeStartupConfig(VersionedStartupConfig& startupConfig, const VersionedStartupConfig& defaults);
+  void MergeStartupConfig(InternalStartupConfig& startupConfig, const InternalStartupConfig& defaults);
 
 private:
-  void InitializeRootDirectories(const VersionedStartupConfig& startupConfig, bool review);
+  void InitializeRootDirectories(const InternalStartupConfig& startupConfig, bool review);
 
 private:
-  void SaveStartupConfig(const VersionedStartupConfig& startupConfig, MiKTeX::Core::RegisterRootDirectoriesOptionSet options);
+  void SaveStartupConfig(const InternalStartupConfig& startupConfig, MiKTeX::Core::RegisterRootDirectoriesOptionSet options);
 
 private:
   bool IsTeXMFReadOnly(unsigned r);
@@ -1008,34 +1009,34 @@ private:
     std::unique_ptr<MiKTeX::Core::Process> process;
   };
 
-  // caching open files
 private:
+  // caching open files
   std::map<const FILE*, InternalOpenFileInfo> openFilesMap;
 
-  // caching path patterns
 private:
+  // caching path patterns
   typedef std::unordered_map<std::string, std::vector<MiKTeX::Util::PathName>> SearchPathDictionary;
 
 private:
   SearchPathDictionary expandedPathPatterns;
 
-  // file access history
 private:
+  // file access history
   std::vector<MiKTeX::Core::FileInfoRecord> fileInfoRecords;
 
-  // true, if we record a file history
 private:
+  // true, if we record a file history
   bool recordingFileNames = false;
 
-  // true, if we record a package history
 private:
+  // true, if we record a package history
   bool recordingPackageNames = false;
 
 private:
   std::ofstream fileNameRecorderStream;
 
-  // package history file
 private:
+  // package history file
   std::string packageHistoryFile;
 
 private:
@@ -1054,9 +1055,10 @@ private:
   std::vector<LanguageInfo_> languages;
 
 private:
-  VersionedStartupConfig initStartupConfig;
+  InternalStartupConfig initStartupConfig;
 
 private:
+  // session initialization options provided by caller
   MiKTeX::Core::Session::InitInfo initInfo;
 
 private:
@@ -1104,8 +1106,8 @@ public:
     return engine;
   }
 
-  // registered root directories
 private:
+  // registered root directories
   std::vector<RootDirectoryInternals> rootDirectories;
 
 private:
@@ -1124,28 +1126,28 @@ private:
   void ConnectToServer();
 #endif
 
-  // index of common data root
 private:
+  // index of common data root
   unsigned commonDataRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
-  // index of user data root
 private:
+  // index of user data root
   unsigned userDataRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
-  // index of common install root
 private:
+  // index of common install root
   unsigned commonInstallRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
-  // index of user install root
 private:
+  // index of user install root
   unsigned userInstallRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
-  // index of common config root
 private:
+  // index of common config root
   unsigned commonConfigRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
-  // index of user config root
 private:
+  // index of user config root
   unsigned userConfigRootIndex = MiKTeX::Core::INVALID_ROOT_INDEX;
 
 private:
@@ -1154,15 +1156,15 @@ private:
 private:
   std::set<std::string> valuesBeingExpanded;
 
+private:
   // the MiKTeXDirect flag:
   // TriState::True if running from a MiKTeXDirect medium
   // TriState::False if not running from a MiKTeXDirect medium
   // TriState::Undetermined otherwise
-private:
  MiKTeX::Configuration::TriState triMiKTeXDirect = MiKTeX::Configuration::TriState::Undetermined;
 
-  // fully qualified path to the running application file
 private:
+  // fully qualified path to the running application file
   MiKTeX::Util::PathName myProgramFile;
   MiKTeX::Util::PathName myProgramFileCanon;
 
@@ -1177,6 +1179,10 @@ private:
   void StartFinishScript(int delay);
 
 private:
+  std::shared_ptr<MiKTeX::Core::FileSystemWatcher> fsWatcher;
+
+private:
+  // indicates whether the session has been initialized
   bool initialized = false;
 
 #if defined(MIKTEX_WINDOWS) && USE_LOCAL_SERVER
