@@ -117,87 +117,6 @@ PathName SessionImpl::GetMyPrefix(bool canonicalized)
   MIKTEX_FATAL_ERROR_2("Cannot derive the path prefix of the running executable.", "bindir", bindir.ToString(), "canonicalized", canonicalized ? "true" : "false");
 }
 
-bool SessionImpl::FindStartupConfigFile(ConfigurationScope scope, PathName& path)
-{
-  string str;
-
-  if (Utils::GetEnvironmentString(scope == ConfigurationScope::Common ? MIKTEX_ENV_COMMON_STARTUP_FILE : MIKTEX_ENV_USER_STARTUP_FILE, str))
-  {
-    path = str;
-    // don't check for existence; it's a fatal error (detected later)
-    // if the environment value is incorrect
-    return true;
-  }
-
-#if USE_WINDOWS_REGISTRY
-  if (winRegistry::TryGetValue(scope, MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_STARTUP_FILE, str))
-  {
-    // don't check for existence; it's a fatal error (detected later)
-    // if the registry value is incorrect
-    path = str;
-    return true;
-  }
-#endif
-
-  StartupConfig defaultStartupConfig = DefaultConfig();
-
-  if (scope == ConfigurationScope::Common)
-  {
-    // try the prefix of the internal bin directory
-    PathName myloc = GetMyLocation(true);
-    RemoveDirectoryDelimiter(myloc.GetData());
-    PathName internalBindir(MIKTEX_PATH_INTERNAL_BIN_DIR);
-    RemoveDirectoryDelimiter(internalBindir.GetData());
-    PathName prefix;
-    if (Utils::GetPathNamePrefix(myloc, internalBindir, prefix))
-    {
-      path = prefix / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
-      if (File::Exists(path))
-      {
-        return true;
-      }
-    }
-    // try the prefix of the bin directory
-    PathName bindir(MIKTEX_PATH_BIN_DIR);
-    RemoveDirectoryDelimiter(bindir.GetData());
-    if (Utils::GetPathNamePrefix(myloc, bindir, prefix))
-    {
-      path = prefix / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
-      if (File::Exists(path))
-      {
-        return true;
-      }
-    }
-    // try /var/lib/miktex-texmf/miktex/config/miktexstartup.ini
-    path = defaultStartupConfig.commonConfigRoot / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
-    if (File::Exists(path))
-    {
-      return true;
-    }
-#if defined(MIKTEX_UNIX) && !defined(MIKTEX_MACOS_BUNDLE)
-    // try /usr/share/miktex-texmf/miktex/config/miktexstartup.ini
-    prefix = GetMyPrefix(true);
-    path = prefix / PathName(MIKTEX_INSTALL_DIR) / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
-    if (File::Exists(path))
-    {
-      return true;
-    }
-#endif
-  }
-
-  if (scope == ConfigurationScope::User)
-  {
-    // try $HOME/.miktex/miktex/config/miktexstartup.ini
-    path = defaultStartupConfig.userConfigRoot / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
-    if (File::Exists(path))
-    {
-      return true;
-    }
-  }
-
-  return false;
-}
-
 bool SessionImpl::IsMiKTeXDirect()
 {
   return initStartupConfig.config == MiKTeXConfiguration::Direct;
@@ -606,29 +525,7 @@ bool SessionImpl::IsAdminMode()
 
 bool SessionImpl::IsSharedSetup()
 {
-  if (isSharedSetup == TriState::Undetermined)
-  {
-    isSharedSetup = GetConfigValue(MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_SHARED_SETUP, ConfigValue(TriState::Undetermined)).GetTriState();
-    if (isSharedSetup == TriState::Undetermined)
-    {
-      string value;
-      isSharedSetup = TryGetConfigValue(MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_LAST_ADMIN_MAINTENANCE, value) ? TriState::True : TriState::Undetermined;
-      if (isSharedSetup == TriState::Undetermined)
-      {
-#if defined(MIKTEX_WINDOWS)
-        isSharedSetup = TriState::False;
-#else
-        PathName myLoc = GetMyLocation(true);
-#if defined(MIKTEX_MACOS_BUNDLE)
-        isSharedSetup = Utils::IsParentDirectoryOf(PathName("/usr"), myLoc) || Utils::IsParentDirectoryOf(PathName("/Applications"), myLoc) ? TriState::True : TriState::False;
-#else
-        isSharedSetup = Utils::IsParentDirectoryOf(PathName("/usr"), myLoc) || Utils::IsParentDirectoryOf(PathName("/opt"), myLoc) ? TriState::True : TriState::False;
-#endif
-#endif
-      }
-    }
-  }
-  return isSharedSetup == TriState::True;
+  return initStartupConfig.isSharedSetup == TriState::True;
 }
 
 void SessionImpl::ConfigureFile(const PathName& pathRel, HasNamedValues* callback)
