@@ -88,6 +88,11 @@ public:
   {
     try
     {
+      if (!cameFrom.Empty())
+      {
+        Directory::SetCurrent(cameFrom);
+        cameFrom = "";
+      }
       if (!path.Empty())
       {
         Delete();
@@ -101,6 +106,10 @@ public:
 public:
   void MIKTEXTHISCALL Delete() override
   {
+    if (!cameFrom.Empty())
+    {
+      MIKTEX_UNEXPECTED();
+    }
     Directory::Delete(path, true);
     path = "";
   }
@@ -117,6 +126,17 @@ public:
     path = "";
   }
 
+public:
+  void MIKTEXTHISCALL SetCurrent() override
+  {
+    if (!cameFrom.Empty())
+    {
+      MIKTEX_UNEXPECTED();
+    }
+    cameFrom.SetToCurrentDirectory();
+    Directory::SetCurrent(path);
+  }
+
 private:
   static bool NameExists(const PathName& name)
   {
@@ -129,6 +149,9 @@ private:
 #endif
   }
 
+private:
+  PathName cameFrom;
+
 private:
   PathName path;
 };
