@@ -1,6 +1,6 @@
 /* internal.h: internal DVI definitions                 -*- C++ -*-
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX DVI Library.
 
@@ -26,8 +26,11 @@
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 
+#include <miktex/Core/BufferSizes>
+#include <miktex/Core/Debug>
 #include <miktex/Core/FileStream>
 #include <miktex/Core/TemporaryFile>
+#include <miktex/Core/Utils>
 #include <miktex/DVI/Dvi>
 #include <miktex/Trace/TraceStream>
 #include <miktex/Trace/Trace>
@@ -35,6 +38,7 @@
 using namespace std;
 using namespace std::string_literals;
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::DVI;
 using namespace MiKTeX::Graphics;
