@@ -68,33 +68,6 @@ SETUPSTATICFUNC(int) ComparePaths(const PathName& path1, const PathName& path2,
 
 BEGIN_INTERNAL_NAMESPACE;
 
-void RemoveEmptyDirectoryChain(const PathName& directory)
-{
-  unique_ptr<DirectoryLister> lister = DirectoryLister::Open(directory);
-  DirectoryEntry dirEntry;
-  bool empty = !lister->GetNext(dirEntry);
-  lister->Close();
-  if (!empty)
-  {
-    return;
-  }
-  FileAttributeSet attributes = File::GetAttributes(directory);
-  if (attributes[FileAttribute::ReadOnly])
-  {
-    attributes -= FileAttribute::ReadOnly;
-    File::SetAttributes(directory, attributes);
-  }
-  Directory::Delete(directory);
-  PathName parentDir(directory);
-  parentDir.CutOffLastComponent();
-  if (parentDir == directory)
-  {
-    return;
-  }
-  // RECURSION
-  RemoveEmptyDirectoryChain(parentDir);
-}
-
 bool Contains(const vector<PathName>& vec, const PathName& pathName)
 {
   for (const PathName& p : vec)
@@ -681,15 +654,18 @@ void SetupServiceImpl::CompleteOptions(bool allowRemoteCalls)
   }
   if (options.Task == SetupTask::CleanUp)
   {
-    options.Config.commonInstallRoot = session->GetSpecialPath(SpecialPath::CommonInstallRoot);
-    options.Config.commonConfigRoot = session->GetSpecialPath(SpecialPath::CommonConfigRoot);
-    options.Config.commonDataRoot = session->GetSpecialPath(SpecialPath::CommonDataRoot);
     if (!session->IsAdminMode())
     {
       options.Config.userInstallRoot = session->GetSpecialPath(SpecialPath::UserInstallRoot);
       options.Config.userConfigRoot = session->GetSpecialPath(SpecialPath::UserConfigRoot);
       options.Config.userDataRoot = session->GetSpecialPath(SpecialPath::UserDataRoot);
     }
+    if (session->IsSharedSetup())
+    {
+      options.Config.commonInstallRoot = session->GetSpecialPath(SpecialPath::CommonInstallRoot);
+      options.Config.commonConfigRoot = session->GetSpecialPath(SpecialPath::CommonConfigRoot);
+      options.Config.commonDataRoot = session->GetSpecialPath(SpecialPath::CommonDataRoot);
+    }
     return;
   }
   if (options.Task == SetupTask::InstallFromLocalRepository)
@@ -1184,7 +1160,7 @@ void SetupServiceImpl::DoCleanUp()
         parent.CutOffLastComponent();
         if (Directory::Exists(parent))
         {
-          RemoveEmptyDirectoryChain(parent);
+          Directory::RemoveEmptyDirectoryChain(parent);
         }
       }
       if (!session->IsAdminMode())
@@ -1193,13 +1169,13 @@ void SetupServiceImpl::DoCleanUp()
         parent.CutOffLastComponent();
         if (Directory::Exists(parent))
         {
-          RemoveEmptyDirectoryChain(parent);
+          Directory::RemoveEmptyDirectoryChain(parent);
         }
         parent = session->GetSpecialPath(SpecialPath::UserConfigRoot);
         parent.CutOffLastComponent();
         if (Directory::Exists(parent))
         {
-          RemoveEmptyDirectoryChain(parent);
+          Directory::RemoveEmptyDirectoryChain(parent);
         }
       }
       if (session->IsAdminMode())
@@ -1208,13 +1184,13 @@ void SetupServiceImpl::DoCleanUp()
         parent.CutOffLastComponent();
         if (Directory::Exists(parent))
         {
-          RemoveEmptyDirectoryChain(parent);
+          Directory::RemoveEmptyDirectoryChain(parent);
         }
         parent = session->GetSpecialPath(SpecialPath::CommonConfigRoot);
         parent.CutOffLastComponent();
         if (Directory::Exists(parent))
         {
-          RemoveEmptyDirectoryChain(parent);
+          Directory::RemoveEmptyDirectoryChain(parent);
         }
       }
     }
@@ -1847,14 +1823,6 @@ void SetupServiceImpl::WriteReport(ostream& s, ReportOptionSet options)
       << "SharedSetup: " << (session->IsSharedSetup() ? "yes" : "no") << "\n"
       << "LinkTargetDirectory: " << session->GetSpecialPath(SpecialPath::LinkTargetDirectory) << "\n"
       << "PathOkay: " << (p.first ? "yes" : "no") << "\n";
-    if (session->IsSharedSetup())
-    {
-      InstallationSummary commonInstallation = packageManager->GetInstallationSummary(false);
-      s << "LastUpdateCheckAdmin: " << FormatTimestamp(commonInstallation.lastUpdateCheck) << "\n";
-      s << "LastUpdateAdmin: " << FormatTimestamp(commonInstallation.lastUpdate) << "\n";
-      s << "LastUpdateDbAdmin: " << FormatTimestamp(commonInstallation.lastUpdateDb) << "\n";
-
-    }
     if (!session->IsAdminMode())
     {
       InstallationSummary userInstallation = packageManager->GetInstallationSummary(true);
@@ -1865,6 +1833,14 @@ void SetupServiceImpl::WriteReport(ostream& s, ReportOptionSet options)
         s << "LastUpdateDb: " << FormatTimestamp(userInstallation.lastUpdateDb) << "\n";
       }
     }
+    if (session->IsSharedSetup())
+    {
+      InstallationSummary commonInstallation = packageManager->GetInstallationSummary(false);
+      s << "LastUpdateCheckAdmin: " << FormatTimestamp(commonInstallation.lastUpdateCheck) << "\n";
+      s << "LastUpdateAdmin: " << FormatTimestamp(commonInstallation.lastUpdate) << "\n";
+      s << "LastUpdateDbAdmin: " << FormatTimestamp(commonInstallation.lastUpdateDb) << "\n";
+
+    }
   }
   if (options[ReportOption::CurrentUser])
   {
@@ -1885,9 +1861,12 @@ void SetupServiceImpl::WriteReport(ostream& s, ReportOptionSet options)
         << "UserConfig: " << session->GetSpecialPath(SpecialPath::UserConfigRoot) << "\n"
         << "UserData: " << session->GetSpecialPath(SpecialPath::UserDataRoot) << "\n";
     }
-    s <<"CommonInstall: " << session->GetSpecialPath(SpecialPath::CommonInstallRoot) << "\n"
-      << "CommonConfig: " << session->GetSpecialPath(SpecialPath::CommonConfigRoot) << "\n"
-      << "CommonData: " << session->GetSpecialPath(SpecialPath::CommonDataRoot) << "\n";
+    if (session->IsSharedSetup())
+    {
+      s << "CommonInstall: " << session->GetSpecialPath(SpecialPath::CommonInstallRoot) << "\n"
+        << "CommonConfig: " << session->GetSpecialPath(SpecialPath::CommonConfigRoot) << "\n"
+        << "CommonData: " << session->GetSpecialPath(SpecialPath::CommonDataRoot) << "\n";
+    }
   }
   if (options[ReportOption::Processes])
   {
