@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2013-2019  Stefan Löffler
+ * Copyright (C) 2013-2021  Stefan Löffler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -14,21 +14,70 @@
 #ifndef PDFDocumentTools_H
 #define PDFDocumentTools_H
 
-#include <QKeyEvent>
-#include <QMouseEvent>
-#include <QPaintEvent>
-#include <QCursor>
-#include <QRubberBand>
-#include <QComboBox>
-#include <QGraphicsView>
-#include <QGraphicsLineItem>
-#include <QGraphicsProxyWidget>
-
 #include "PDFBackend.h"
 
 #ifdef DEBUG
   #include <QDebug>
 #endif
+#include <QComboBox>
+#include <QCursor>
+#include <QGraphicsLineItem>
+#include <QGraphicsProxyWidget>
+#include <QGraphicsView>
+#include <QKeyEvent>
+#include <QMouseEvent>
+#include <QPaintEvent>
+#include <QPointer>
+#include <QRubberBand>
+
+// Class representing an owning pointer to a QObject
+// When an OwningQObjectPointer is destroyed, the QObject it points to is
+// destroyed as well. Unlike std::unique_ptr, OwningQObjectPointer gracefully
+// handles the case that the managed QObject is destroyed externally (e.g. by
+// calling QObject::deleteLater()).
+template <class T>
+class OwningQObjectPointer
+{
+  bool m_deleteLater;
+  QPointer<T> m_p;
+
+  OwningQObjectPointer(const OwningQObjectPointer &) = delete;
+  OwningQObjectPointer<T> & operator=(const OwningQObjectPointer &) = delete;
+public:
+  OwningQObjectPointer(T * p, const bool deleteLater = false) : m_deleteLater(deleteLater), m_p(p) {}
+  ~OwningQObjectPointer() noexcept {
+    if (!m_deleteLater) {
+      delete m_p;
+    }
+    else if(!m_p.isNull()) {
+      m_p->deleteLater();
+    }
+  }
+  OwningQObjectPointer(OwningQObjectPointer &&) = default;
+  OwningQObjectPointer<T> & operator=(OwningQObjectPointer &&) = default;
+  OwningQObjectPointer<T> & operator=(T * p) {
+    QPointer<T> oldP = m_p;
+    m_p = p;
+    if (!m_deleteLater) {
+      delete oldP;
+    }
+    else if (!oldP.isNull()) {
+      oldP->deleteLater();
+    }
+    return *this;
+  }
+
+  T* data() const { return m_p.data(); }
+  T* operator->() const { return data(); }
+  T& operator*() const { return *data(); }
+  operator T*() const { return data(); }
+  bool isNull() const { return m_p.isNull(); }
+  void clear() { m_p.clear(); }
+
+  bool usesDeleteLater() const { return m_deleteLater; }
+  void setDeleteLater(const bool deleteLater = true) { m_deleteLater = deleteLater; }
+};
+
 
 namespace QtPDF {
 
@@ -44,10 +93,17 @@ public:
   enum Type { Tool_None, Tool_MagnifyingGlass, Tool_ZoomIn, Tool_ZoomOut, Tool_MarqueeZoom, Tool_Move, Tool_ContextMenu, Tool_ContextClick, Tool_Measure, Tool_Select };
   AbstractTool(PDFDocumentView * parent) : _parent(parent), _cursor(QCursor(Qt::ArrowCursor)) { }
   virtual ~AbstractTool() = default;
-  
+
   virtual Type type() const { return Tool_None; }
-  virtual bool operator==(const AbstractTool & o) { return (type() == o.type()); }
+  virtual bool operator==(const AbstractTool & o) const { return (type() == o.type()); }
 protected:
+  // Copy/move c'tor and assignment operators should be protected to avoid
+  // use outside the inheritance tree (which could result in slicing)
+  AbstractTool(const AbstractTool &) = default;
+  AbstractTool(AbstractTool &&) = default;
+  AbstractTool & operator=(AbstractTool &) = default;
+  AbstractTool & operator=(AbstractTool &&) = default;
+
   virtual void arm();
   virtual void disarm();
 
@@ -61,7 +117,7 @@ protected:
   virtual void mouseReleaseEvent(QMouseEvent * event);
   virtual void paintEvent(QPaintEvent * event) { Q_UNUSED(event) }
 
-  PDFDocumentView * _parent;
+  QPointer<PDFDocumentView> _parent;
   QCursor _cursor;
 };
 
@@ -117,7 +173,7 @@ protected:
 
   virtual void hide();
 
-  PDFDocumentMagnifierView * _magnifier;
+  OwningQObjectPointer<PDFDocumentMagnifierView> _magnifier;
   bool _started;
 };
 
@@ -136,7 +192,7 @@ protected:
 
   bool _started;
   QPoint _startPos;
-  QRubberBand * _rubberBand;
+  OwningQObjectPointer<QRubberBand> _rubberBand;
 };
 
 class Move : public AbstractTool
@@ -185,7 +241,7 @@ protected:
 
   // to be called from friend class Measure during creation of the MeasureLine
   void mouseMove(const QPointF scenePos, const Qt::KeyboardModifiers modifiers);
-  
+
   // 1 for the first handle, 2 for the second handle
   int _pt;
 };
@@ -195,7 +251,6 @@ class MeasureLine : public QGraphicsLineItem
   friend class Measure;
 public:
   MeasureLine(QGraphicsView * primaryView, QGraphicsItem * parent = nullptr);
-  ~MeasureLine() override = default;
 
   void setLine(qreal x1, qreal y1, qreal x2, qreal y2) { setLine(QLineF(x1, y1, x2, y2)); }
   void setLine(QPointF p1, QPointF p2) { setLine(QLineF(p1, p2)); }
@@ -210,8 +265,13 @@ protected:
   void mousePressEvent(QGraphicsSceneMouseEvent * event) override;
   void mouseMoveEvent(QGraphicsSceneMouseEvent * event) override;
 
+  // _measureBox will be embedded in _measureBoxProxy, which in turn will be
+  // parented to this object; so they will all be destroyed automatically by
+  // ~QGraphicsItem()
   QComboBox * _measureBox;
   QGraphicsProxyWidget * _measureBoxProxy;
+  // _grip1 & _grip2 are raw pointers, but they are parented to this object, so
+  // they will automatically be destroyed by ~QGraphicsItem()
   MeasureLineGrip * _grip1, * _grip2;
   QMap<QString, float> _measures;
   QGraphicsView * _primaryView;
@@ -230,6 +290,8 @@ protected:
   void keyPressEvent(QKeyEvent * event) override;
   void keyReleaseEvent(QKeyEvent * event) override;
 
+  // _measureLine will immediately be added to a QGraphicsScene, which takes
+  // ownership. So we don't need a std::unqiue_ptr or similar
   MeasureLine * _measureLine;
   bool _started;
   QPoint _startPos;
@@ -259,7 +321,6 @@ class Select : public AbstractTool
   friend class QtPDF::PDFDocumentView;
 public:
   Select(PDFDocumentView * parent);
-  ~Select() override;
   Type type() const override { return Tool_Select; }
 
   QColor highlightColor() const { return _highlightColor; }
@@ -275,7 +336,7 @@ protected:
   void keyPressEvent(QKeyEvent * event) override;
 
   void resetBoxes(const int pageNum = -1);
-  // Call this to notify Select that the page graphics item it has been working 
+  // Call this to notify Select that the page graphics item it has been working
   // on has been destroyed so all pointers to graphics items should be
   // invalidated
   void pageDestroyed();
@@ -286,15 +347,19 @@ protected:
 
   bool _cursorOverBox;
   QPointF _startPos;
+  // _highlightPath will immediately be added to a QGraphicsScene, which takes
+  // ownership. So we don't need a std::unqiue_ptr or similar
   QGraphicsPathItem * _highlightPath;
   MouseMode _mouseMode;
-  QRubberBand * _rubberBand;
+  OwningQObjectPointer<QRubberBand> _rubberBand;
   QColor _highlightColor;
 
   int _pageNum;
   QList<Backend::Page::Box> _boxes;
   int _startBox, _startSubbox;
 #ifdef DEBUG
+  // All elements of _displayBoxes are automatically added to a QGraphicsScene,
+  // which takes ownership. So we don't need a std::unqiue_ptr or similar
   QList<QGraphicsRectItem*> _displayBoxes;
 #endif
 };
