@@ -926,7 +926,7 @@ void TWApp::setDefaultPaths()
 #if defined(MIKTEX)
           {
             auto session = MiKTeX::Core::Session::Get();
-            MiKTeX::Util::PathName d = session->GetSpecialPath(MiKTeX::Configuration::SpecialPath::CommonInstallRoot) / MiKTeX::Util::PathName(MIKTEX_PATH_BIN_DIR);
+            MiKTeX::Util::PathName d = session->GetSpecialPath(MiKTeX::Configuration::SpecialPath::InstallRoot) / MiKTeX::Util::PathName(MIKTEX_PATH_BIN_DIR);
             QString dir = QString::fromUtf8(d.GetData());
             if (!binaryPaths->contains(dir))
             {
