@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2007-2019  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
+	Copyright (C) 2007-2021  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,13 +22,22 @@
 #ifndef TWApp_H
 #define TWApp_H
 
-#include <QApplication>
-#include <QList>
 #include <QAction>
-#include <QSettings>
+#include <QApplication>
 #include <QClipboard>
-#include <QVariant>
 #include <QHash>
+#include <QList>
+#if defined Q_OS_DARWIN
+#include <QMenu>
+#include <QMenuBar>
+#endif
+#include <QSettings>
+#include <QString>
+#include <QTextCodec>
+#include <QVariant>
+
+class Engine;
+class TWScriptManager;
 
 #if defined(Q_OS_WIN)
 #define PATH_LIST_SEP   ";"
@@ -42,24 +51,8 @@
 #endif
 #endif
 
-#ifndef TW_BUILD_ID
-#define TW_BUILD_ID unknown build
-#endif
-#define STRINGIFY_2(s) #s
-#define STRINGIFY(s) STRINGIFY_2(s)
-#if !defined(MIKTEX)
-#define TW_BUILD_ID_STR STRINGIFY(TW_BUILD_ID)
-#endif
-
 #define DEFAULT_ENGINE_NAME "pdfLaTeX"
 
-class QString;
-class QMenu;
-class QMenuBar;
-
-class Engine;
-class TWScriptManager;
-
 // general constants used by multiple document types
 const int kStatusMessageDuration = 3000;
 const int kNewWindowOffset = 32;
@@ -71,6 +64,9 @@ class TWApp : public QApplication
 	// window positioning utilities
 	typedef void (WindowArrangementFunction)(const QWidgetList& windows, const QRect& bounds);
 
+	// FIXME: Required for functor-access to private slot openRecentFile()
+	friend class TWUtils;
+
 public:
 	TWApp(int &argc, char **argv);
 	~TWApp() override;
@@ -80,13 +76,13 @@ public:
 	void addToRecentFiles(const QMap<QString,QVariant>& fileProperties);
 
 	void emitHighlightLineOptionChanged();
-	
+
 	QMap<QString,QVariant> getFileProperties(const QString& path);
-	
+
 	void setBinaryPaths(const QStringList& paths);
 	void setEngineList(const QList<Engine>& engines);
 
-	const QStringList getBinaryPaths(QStringList& sysEnv);
+	const QStringList getBinaryPaths();
 	// runtime paths, including $PATH;
 	// also modifies passed-in sysEnv to include paths from prefs
 	QString findProgram(const QString& program, const QStringList& binPaths);
@@ -94,25 +90,25 @@ public:
 	const QStringList getPrefsBinaryPaths(); // only paths from prefs
 	const QList<Engine> getEngineList();
 	void saveEngineList();
-	
+
 	const Engine getNamedEngine(const QString& name);
 	const Engine getDefaultEngine();
 	void setDefaultEngine(const QString& name);
 
 	void setDefaultPaths();
 	void setDefaultEngineList();
-	
+
 	QTextCodec *getDefaultCodec();
 	void setDefaultCodec(QTextCodec *codec);
 
 	void openUrl(const QUrl& url);
 
 	static TWApp *instance();
-	
-	QString getPortableLibPath() const { return portableLibPath; }
+
+	static QStringList getTranslationList();
 
 	TWScriptManager* getScriptManager() { return scriptManager; }
-	
+
 #if defined(Q_OS_WIN)
 	static QString GetWindowsVersionString();
 	static unsigned int GetWindowsVersion();
@@ -122,7 +118,7 @@ public:
 	QMap<QString, QVariant> openFileFromScript(const QString& fileName, QObject * scriptApiObj, const int pos = -1, const bool askUser = false);
 
 	Q_INVOKABLE QList<QVariant> getOpenWindows() const;
-	
+
 	// return the version of Tw (0xMMNNPP)
 	Q_INVOKABLE
 	static int getVersion();
@@ -143,33 +139,37 @@ public:
 	Q_INVOKABLE void unsetGlobal(const QString& key) { m_globals.remove(key); }
 	Q_INVOKABLE bool hasGlobal(const QString& key) const { return m_globals.contains(key); }
 	Q_INVOKABLE QVariant getGlobal(const QString& key) const { return m_globals[key]; }
-	
+
 #if defined(Q_OS_DARWIN)
+	void recreateSpecialMenuItems();
 private:
 	// on the Mac only, we have a top-level app menu bar, including its own copy of the recent files menu
 	QMenuBar *menuBar;
 
-	QMenu *menuFile;
-	QAction *actionNew;
-	QAction *actionNew_from_Template;
-	QAction *actionOpen;
-	QAction *actionPreferences;
-	QAction *actionQuit;
+	QMenu *menuFile{nullptr};
+	QAction *actionNew{nullptr};
+	QAction *actionNew_from_Template{nullptr};
+	QAction *actionOpen{nullptr};
+	QAction *actionPreferences{nullptr};
+	QAction *actionQuit{nullptr};
 
-	QMenu *menuRecent;
-	QAction *actionClear_Recent_Files;
+	QMenu *menuRecent{nullptr};
+	QAction *actionClear_Recent_Files{nullptr};
 	QList<QAction*> recentFileActions;
 
-	QMenu *menuHelp;
-	QAction *aboutAction;
-	QAction *homePageAction;
-	QAction *mailingListAction;
+	QMenu *menuHelp{nullptr};
+	QAction *aboutAction{nullptr};
+	QAction *homePageAction{nullptr};
+	QAction *mailingListAction{nullptr};
 #endif
 
 public slots:
 	void bringToFront();
 	QObject* openFile(const QString& fileName, const int pos = -1);
 
+	void preferences();
+
+	void clearRecentFiles();
 	// called by documents when they load a file
 	void updateRecentFileActions();
 
@@ -188,7 +188,7 @@ public slots:
 	void openHelpFile(const QString& helpDirName);
 
 	void applyTranslation(const QString& locale);
-	
+
 	void maybeQuit();
 
 	void updateScriptsList();
@@ -197,7 +197,8 @@ public slots:
 	void about();
 	void doResourcesDialog() const;
 	QObject * newFile() const;
-	void open();
+	QObject * newFromTemplate() const;
+	void open(const QString & defaultFilter = {});
 	void stackWindows();
 	void tileWindows();
 
@@ -207,19 +208,19 @@ public slots:
         void aboutMiKTeX();
         void UnloadFileNameDatabase();
 #endif
-	
+
 signals:
 	// emitted in response to updateRecentFileActions(); documents can listen to this if they have a recent files menu
 	void recentFileActionsChanged();
 
 	// emitted when the window list may have changed, so documents can update their window menu
 	void windowListChanged();
-	
+
 	// emitted when the engine list is changed from Preferences, so docs can update their menus
 	void engineListChanged();
-	
+
 	void scriptListChanged();
-	
+
 	void syncPdf(const QString& sourceFile, int lineNo, int col, bool activatePreview);
 
 	void hideFloatersExcept(QWidget* theWindow);
@@ -229,10 +230,7 @@ signals:
 	void highlightLineOptionChanged();
 
 private slots:
-	void clearRecentFiles();
-	QObject * newFromTemplate() const;
 	void openRecentFile();
-	void preferences();
 
 	void changeLanguage();
 
@@ -243,7 +241,7 @@ protected:
 
 private:
 	void init();
-	
+
 	void arrangeWindows(WindowArrangementFunction func);
 
 	int recentFilesLimit;
@@ -254,15 +252,13 @@ private:
 	QStringList *defaultBinPaths;
 	QList<Engine> *engineList;
 	int defaultEngineIndex;
-	QString portableLibPath;
 
 	QList<QTranslator*> translators;
-	
+
 	TWScriptManager *scriptManager;
 
 	QHash<QString, QVariant> m_globals;
-	QList<QTextCodec*> customTextCodecs;
-	
+
 	static TWApp *theAppInstance;
 };
 
@@ -277,7 +273,7 @@ class TWDocumentOpenEvent : public QEvent
 public:
 	static const QEvent::Type type;
 	explicit TWDocumentOpenEvent(const QString & filename, const int pos = 0) : QEvent(type), filename(filename), pos(pos) { }
-	
+
 	QString filename;
 	int pos;
 };
