@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2019  Stefan Löffler
+	Copyright (C) 2019-2020  Stefan Löffler
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 	For links to further information, or to contact the authors,
 	see <http://www.tug.org/texworks/>.
 */
+
 #if defined(MIKTEX)
 #include <fmt/format.h>
 #include <fmt/ostream.h>
@@ -25,19 +26,21 @@
 #include <miktex/miktex-texworks.hpp>
 #endif
 #include "document/SpellChecker.h"
+
 #include "TWUtils.h" // for TWUtils::getLibraryPath
+#include "utils/ResourcesLibrary.h"
 
 #include <hunspell.h>
 
 namespace Tw {
 namespace Document {
 
-QHash<QString, QString> * SpellChecker::dictionaryList = nullptr;
+QMultiHash<QString, QString> * SpellChecker::dictionaryList = nullptr;
 QHash<const QString,SpellChecker::Dictionary*> * SpellChecker::dictionaries = nullptr;
 SpellChecker * SpellChecker::_instance = new SpellChecker();
 
 // static
-QHash<QString, QString> * SpellChecker::getDictionaryList(const bool forceReload /* = false */)
+QMultiHash<QString, QString> * SpellChecker::getDictionaryList(const bool forceReload /* = false */)
 {
 	if (dictionaryList) {
 		if (!forceReload)
@@ -45,14 +48,14 @@ QHash<QString, QString> * SpellChecker::getDictionaryList(const bool forceReload
 		delete dictionaryList;
 	}
 
-	dictionaryList = new QHash<QString, QString>();
-	const QStringList dirs = TWUtils::getLibraryPaths(QStringLiteral("dictionaries"));
+	dictionaryList = new QMultiHash<QString, QString>();
+	const QStringList dirs = Tw::Utils::ResourcesLibrary::getLibraryPaths(QStringLiteral("dictionaries"));
 	foreach (QDir dicDir, dirs) {
 		foreach (QFileInfo dicFileInfo, dicDir.entryInfoList(QStringList(QString::fromLatin1("*.dic")),
 					QDir::Files | QDir::Readable, QDir::Name | QDir::IgnoreCase)) {
 			QFileInfo affFileInfo(dicFileInfo.dir(), dicFileInfo.completeBaseName() + QLatin1String(".aff"));
 			if (affFileInfo.isReadable())
-				dictionaryList->insertMulti(dicFileInfo.canonicalFilePath(), dicFileInfo.completeBaseName());
+				dictionaryList->insert(dicFileInfo.canonicalFilePath(), dicFileInfo.completeBaseName());
 		}
 	}
 #if defined(MIKTEX)
@@ -90,7 +93,7 @@ SpellChecker::Dictionary * SpellChecker::getDictionary(const QString& language)
 	if (dictionaries->contains(language))
 		return dictionaries->value(language);
 
-	const QStringList dirs = TWUtils::getLibraryPaths(QStringLiteral("dictionaries"));
+	const QStringList dirs = Tw::Utils::ResourcesLibrary::getLibraryPaths(QStringLiteral("dictionaries"));
 	foreach (QDir dicDir, dirs) {
 		QFileInfo affFile(dicDir, language + QLatin1String(".aff"));
 		QFileInfo dicFile(dicDir, language + QLatin1String(".dic"));
@@ -169,7 +172,7 @@ bool SpellChecker::Dictionary::isWordCorrect(const QString & word) const
 QList<QString> SpellChecker::Dictionary::suggestionsForWord(const QString & word) const
 {
 	QList<QString> suggestions;
-	char ** suggestionList;
+	char ** suggestionList{nullptr};
 
 	int numSuggestions = Hunspell_suggest(_hunhandle, &suggestionList, _codec->fromUnicode(word).data());
 	suggestions.reserve(numSuggestions);
