@@ -1,6 +1,6 @@
 /* miktexsetup.cpp:
 
-   Copyright (C) 2014-2020 Christian Schenk
+   Copyright (C) 2014-2021 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -54,6 +54,10 @@
 #include <miktex/Util/StringUtil>
 #include <miktex/Wrappers/PoptWrapper>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/ConsoleCodePageSwitcher>
+#endif
+
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::Setup;
@@ -551,14 +555,6 @@ void Application::PrintInfo()
 
 void Application::Main(int argc, const char** argv)
 {
-#if defined(MIKTEX_WINDOWS)
-  UINT activeOutputCodePage = GetConsoleOutputCP();
-  if (activeOutputCodePage != CP_UTF8)
-  {
-    SetConsoleOutputCP(CP_UTF8);
-  }
-#endif
-
   programName = PathName(argv[0]).GetFileName().ToString();
 
   Session::InitInfo initInfo;
@@ -1028,6 +1024,9 @@ extern "C" void Application::SignalHandler(int signalToBeHandled)
 
 int MAIN(int argc, MAINCHAR** argv)
 {
+#if defined(MIKTEX_WINDOWS)
+  ConsoleCodePageSwitcher cpSwitcher;
+#endif
   int retCode = 0;
   try
   {
