@@ -1,6 +1,6 @@
 /* MakeUtility.h:                                       -*- C++ -*-
 
-   Copyright (C) 1998-2020 Christian Schenk
+   Copyright (C) 1998-2021 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -35,12 +35,16 @@
 #include <miktex/Core/CommandLineBuilder>
 #include <miktex/Core/Directory>
 #include <miktex/Core/File>
-#include <miktex/Core/PathName>
+#include <miktex/Util/PathName>
 #include <miktex/Core/Paths>
 #include <miktex/Core/Process>
 #include <miktex/Core/Quoter>
 #include <miktex/Core/Session>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/ConsoleCodePageSwitcher>
+#endif
+
 #include <miktex/Util/StringUtil>
 
 #define OUT__ (stdoutStderr ? std::cerr : std::cout)
@@ -54,7 +58,7 @@
 
 inline bool HasPrefix(const std::string& s1, const std::string& s2)
 {
-  return MiKTeX::Core::PathName::Compare(MiKTeX::Core::PathName(s1), MiKTeX::Core::PathName(s2), s2.length()) == 0;
+  return MiKTeX::Util::PathName::Compare(MiKTeX::Util::PathName(s1), MiKTeX::Util::PathName(s2), s2.length()) == 0;
 }
 
 class ProcessOutputTrash :
@@ -142,10 +146,10 @@ protected:
         session->SetAdminMode(true);
         break;
       case 'D':
-        this->EnableInstaller(MiKTeX::Core::TriState::False);
+        this->EnableInstaller(MiKTeX::Configuration::TriState::False);
         break;
       case 'E':
-        this->EnableInstaller(MiKTeX::Core::TriState::True);
+        this->EnableInstaller(MiKTeX::Configuration::TriState::True);
         break;
       case 'h':
         Usage();
@@ -184,10 +188,10 @@ protected:
   }
   
 protected:
-  bool RunProcess(const char* exeName, const std::vector<std::string>& arguments, const MiKTeX::Core::PathName& workingDirectory)
+  bool RunProcess(const char* exeName, const std::vector<std::string>& arguments, const MiKTeX::Util::PathName& workingDirectory)
   {
     // find the executable; make sure it contains no blanks
-    MiKTeX::Core::PathName exe;
+    MiKTeX::Util::PathName exe;
     if (!session->FindFile(exeName, MiKTeX::Core::FileType::EXE, exe))
     {
       FatalError(fmt::format(T_("The application file {0} could not be found."), Q_(exeName)));
@@ -202,10 +206,10 @@ protected:
 
     switch (GetEnableInstaller())
     {
-    case MiKTeX::Core::TriState::False:
+    case MiKTeX::Configuration::TriState::False:
       allArgs.push_back("--miktex-disable-installer");
       break;
-    case MiKTeX::Core::TriState::True:
+    case MiKTeX::Configuration::TriState::True:
       allArgs.push_back("--miktex-enable-installer");
       break;
     default:
@@ -246,7 +250,7 @@ protected:
   }
 
 protected:
-  bool RunMETAFONT(const char* name, const char* mode, const char* mag, const MiKTeX::Core::PathName& workingDirectory)
+  bool RunMETAFONT(const char* name, const char* mode, const char* mag, const MiKTeX::Util::PathName& workingDirectory)
   {
     std::vector<std::string> arguments;
     arguments.push_back(fmt::format("--undump={}", "mf"));
@@ -265,7 +269,7 @@ protected:
       return true;
     }
     Verbose(T_("METAFONT failed for some reason"));
-    MiKTeX::Core::PathName pathLogFile = workingDirectory / MiKTeX::Core::PathName(name);
+    MiKTeX::Util::PathName pathLogFile = workingDirectory / MiKTeX::Util::PathName(name);
     pathLogFile.AppendExtension(".log");
     bool noError = true;
     size_t nStrangePaths = 0;
@@ -293,7 +297,7 @@ protected:
   }
   
 protected:
-  void Install(const MiKTeX::Core::PathName& source, const MiKTeX::Core::PathName& dest)
+  void Install(const MiKTeX::Util::PathName& source, const MiKTeX::Util::PathName& dest)
   {
     PrintOnly(fmt::format("cp {} {}", Q_(source), Q_(dest)));
     PrintOnly("initexmf --update-fndb");
@@ -305,15 +309,15 @@ protected:
   }
 
 protected:
-  MiKTeX::Core::PathName CreateDirectoryFromTemplate(const std::string& templ)
+  MiKTeX::Util::PathName CreateDirectoryFromTemplate(const std::string& templ)
   {
-    MiKTeX::Core::PathName path;
+    MiKTeX::Util::PathName path;
     const char* lpszTemplate = templ.c_str();
     if (lpszTemplate[0] == '%'
         && lpszTemplate[1] == 'R'
         && MiKTeX::Util::PathNameUtil::IsDirectoryDelimiter(lpszTemplate[2]))
     {
-      path = session->GetSpecialPath(MiKTeX::Core::SpecialPath::DataRoot);
+      path = session->GetSpecialPath(MiKTeX::Configuration::SpecialPath::DataRoot);
       path /= lpszTemplate + 3;
     }
     else
@@ -398,7 +402,7 @@ protected:
   bool stdoutStderr = true;
 
 protected:
-  MiKTeX::Core::PathName destinationDirectory;
+  MiKTeX::Util::PathName destinationDirectory;
 
 protected:
   std::string name;
