@@ -1,6 +1,6 @@
 /* mkfntmap.cpp:
 
-   Copyright (C) 2002-2020 Christian Schenk
+   Copyright (C) 2002-2021 Christian Schenk
 
    This file is part of MkFntMap.
 
@@ -30,6 +30,7 @@
 using namespace std;
 
 using namespace MiKTeX::App;
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Util;
 using namespace MiKTeX::Wrappers;
@@ -1365,6 +1366,9 @@ void MakeFontMapApp::Run()
 
 int MAIN(int argc, MAINCHAR** argv)
 {
+#if defined(MIKTEX_WINDOWS)
+  ConsoleCodePageSwitcher cpSwitcher;
+#endif
   MakeFontMapApp app;
   try
   {
@@ -1392,14 +1396,14 @@ int MAIN(int argc, MAINCHAR** argv)
   }
   catch (const MiKTeXException& ex)
   {
-    Application::Sorry(THE_NAME_OF_THE_GAME, ex);
+    app.Sorry(THE_NAME_OF_THE_GAME, ex);
     logger = nullptr;
     ex.Save();
     return 1;
   }
   catch (const exception& ex)
   {
-    Application::Sorry(THE_NAME_OF_THE_GAME, ex);
+    app.Sorry(THE_NAME_OF_THE_GAME, ex);
     logger = nullptr;
     return 1;
   }
