@@ -1,6 +1,6 @@
 /* mcd.cpp: MiKTeX compiler driver
 
-   Copyright (C) 1998-2020 Christian Schenk
+   Copyright (C) 1998-2021 Christian Schenk
 
    Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2001,
    2002, 2003, 2004, 2005 Free Software Foundation, Inc.
@@ -65,6 +65,10 @@
 #include <miktex/Util/inliners.h>
 #include <miktex/Wrappers/PoptWrapper>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/ConsoleCodePageSwitcher>
+#endif
+
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 
@@ -1727,7 +1731,7 @@ void Driver::RunViewer()
 void Driver::Run()
 {
   ExpandMacros();
-  Application::CheckCancel();
+  app->CheckCancel();
   InsertCommands();
 
   SetIncludeDirectories();
@@ -1742,7 +1746,7 @@ void Driver::Run()
 
   for (int i = 0; i < options->maxIterations; ++i)
   {
-    Application::CheckCancel();
+    app->CheckCancel();
     vector<string> idxFiles;
     GetAuxFiles(previousAuxFiles, &idxFiles);
     if (!previousAuxFiles.empty())
@@ -1753,10 +1757,10 @@ void Driver::Run()
     RunBibTeX();
     if (idxFiles.size() > 0)
     {
-      Application::CheckCancel();
+      app->CheckCancel();
       RunIndexGenerator(idxFiles);
     }
-    Application::CheckCancel();
+    app->CheckCancel();
     RunTeX();
     if (Ready())
     {
@@ -2027,6 +2031,10 @@ const struct poptOption optionTable[] = {
 
 void McdApp::Run(int argc, const char** argv)
 {
+#if defined(MIKTEX_WINDOWS)
+  ConsoleCodePageSwitcher cpSwitcher;
+#endif
+
   Session::InitInfo initInfo(argv[0]);
   vector<const char*> newargv(&argv[0], &argv[argc + 1]);
   ExamineArgs(newargv, initInfo);
@@ -2257,14 +2265,14 @@ int MAIN(int argc, MAINCHAR* argv[])
 
   catch (const MiKTeXException& ex)
   {
-    Application::Sorry(THE_NAME_OF_THE_GAME, ex);
+    app.Sorry(THE_NAME_OF_THE_GAME, ex);
     ex.Save();
     return 1;
   }
 
   catch (const exception& ex)
   {
-    Application::Sorry(THE_NAME_OF_THE_GAME, ex);
+    app.Sorry(THE_NAME_OF_THE_GAME, ex);
     return 1;
   }
 
